@@ -27,3 +27,15 @@ client = BlobServiceClient.from_connection_string(AZURE_BLOB_CONNECTION_STRING)
 # if AZURE_BLOB_ACCOUNT_NAME is equal to "devstoreaccount1", all Azure URLs for objects within that account
 # will start with "azfs://devstoreaccount1/"
 AZURE_BLOB_ACCOUNT_NAME = client.account_name
+
+# Account key of the account. Used to sign SAS URLs.
+AZURE_BLOB_ACCOUNT_KEY = client.credential.account_key
+
+# Container name where bundles are stored.
+AZURE_BLOB_CONTAINER_NAME = "bundles"
+
+# Set to True if using Azurite.
+LOCAL_USING_AZURITE = "http://azurite" in AZURE_BLOB_CONNECTION_STRING
+
+# HTTP endpoint used to directly access Blob Storage. Used to generate SAS URLs.
+AZURE_BLOB_HTTP_ENDPOINT = f"http://localhost:10000/{AZURE_BLOB_ACCOUNT_NAME}" if LOCAL_USING_AZURITE else f"https://{AZURE_BLOB_ACCOUNT_NAME}.blob.core.windows.net"
