@@ -1,26 +1,31 @@
 import copy
-from collections import defaultdict
 import datetime
 import logging
 import os
 import random
 import re
+import shutil
 import sys
+import tempfile
 import threading
 import time
 import traceback
+
+from apache_beam.io.filesystems import FileSystems
+from collections import defaultdict
 from typing import List
 
 from codalab.objects.permission import (
     check_bundles_have_read_permission,
     check_bundle_have_run_permission,
 )
-from codalab.common import NotFoundError, PermissionError
+from codalab.common import NotFoundError, PermissionError, parse_linked_bundle_url
 from codalab.lib import bundle_util, formatting, path_util
 from codalab.server.worker_info_accessor import WorkerInfoAccessor
 from codalab.worker.file_util import remove_path
+from codalab.worker.un_tar_directory import un_tar_directory
 from codalab.worker.bundle_state import State, RunResources
-
+from codalab.worker.download_util import BundleTarget
 
 logger = logging.getLogger(__name__)
 
@@ -37,6 +42,15 @@ MINIMUM_REQUEST_MEMORY_BYTES = 4 * 1024 * 1024
 DISK_QUOTA_SLACK_BYTES = 0.5 * 1024 * 1024 * 1024
 
 
+def normpath(path):
+    """Performs os.path.normpath on a path if it is on the filesystem, but if it is on Beam,
+    doesn't do anything to the path.
+    """
+    if parse_linked_bundle_url(path).uses_beam:
+        return path
+    return os.path.normpath(path)
+
+
 class BundleManager(object):
     """
     Assigns run bundles to workers and makes make bundles.
@@ -52,6 +66,7 @@ class BundleManager(object):
         self._worker_model = codalab_manager.worker_model()
         self._bundle_store = codalab_manager.bundle_store()
         self._upload_manager = codalab_manager.upload_manager()
+        self._download_manager = codalab_manager.download_manager()
 
         self._exiting_lock = threading.Lock()
         self._exiting = False
@@ -205,51 +220,70 @@ class BundleManager(object):
         try:
             bundle_link_url = getattr(bundle.metadata, "link_url", None)
             bundle_location = bundle_link_url or self._bundle_store.get_bundle_location(bundle.uuid)
-            path = os.path.normpath(bundle_location)
+
+            path = normpath(bundle_location)
 
             deps = []
             parent_bundle_link_urls = self._model.get_bundle_metadata(
                 [dep.parent_uuid for dep in bundle.dependencies], "link_url"
             )
-            for dep in bundle.dependencies:
-                parent_bundle_link_url = parent_bundle_link_urls.get(dep.parent_uuid)
-                try:
-                    parent_bundle_path = parent_bundle_link_url or os.path.normpath(
-                        self._bundle_store.get_bundle_location(dep.parent_uuid)
-                    )
-                except NotFoundError:
-                    raise Exception(
-                        'Invalid dependency %s'
-                        % (path_util.safe_join(dep.parent_uuid, dep.parent_path))
-                    )
-                # TODO(Ashwin): make this logic non-fs specific.
-                dependency_path = os.path.normpath(
-                    os.path.join(parent_bundle_path, dep.parent_path)
-                )
-                if not dependency_path.startswith(parent_bundle_path) or (
-                    not os.path.islink(dependency_path) and not os.path.exists(dependency_path)
-                ):
-                    raise Exception(
-                        'Invalid dependency %s'
-                        % (path_util.safe_join(dep.parent_uuid, dep.parent_path))
-                    )
+            with tempfile.TemporaryDirectory() as tempdir:
+                for dep in bundle.dependencies:
+                    parent_bundle_link_url = parent_bundle_link_urls.get(dep.parent_uuid)
+                    try:
+                        parent_bundle_path = parent_bundle_link_url or normpath(
+                            self._bundle_store.get_bundle_location(dep.parent_uuid)
+                        )
+                    except NotFoundError:
+                        raise Exception(
+                            'Invalid dependency %s'
+                            % (path_util.safe_join(dep.parent_uuid, dep.parent_path))
+                        )
+                    dependency_path = normpath(os.path.join(parent_bundle_path, dep.parent_path))
+                    if not dependency_path.startswith(parent_bundle_path) or (
+                        not os.path.islink(dependency_path)
+                        and not FileSystems.exists(dependency_path.rstrip("/"))
+                    ):
+                        raise Exception(
+                            'Invalid dependency %s'
+                            % (path_util.safe_join(dep.parent_uuid, dep.parent_path))
+                        )
 
-                child_path = os.path.normpath(os.path.join(path, dep.child_path))
-                if not child_path.startswith(path):
-                    raise Exception('Invalid key for dependency: %s' % (dep.child_path))
+                    child_path = normpath(os.path.join(path, dep.child_path))
+                    if not child_path.startswith(path):
+                        raise Exception('Invalid key for dependency: %s' % (dep.child_path))
 
-                deps.append((dependency_path, child_path))
+                    # If source path is on Azure Blob Storage, we should download it to a temporary local directory first.
+                    if parse_linked_bundle_url(dependency_path).uses_beam:
+                        dependency_path = os.path.join(tempdir, dep.parent_uuid)
 
-            remove_path(path)
+                        target_info = self._download_manager.get_target_info(
+                            BundleTarget(dep.parent_uuid, dep.parent_path), 0
+                        )
+                        target = target_info['resolved_target']
 
-            if len(deps) == 1 and deps[0][1] == path:
-                path_util.copy(deps[0][0], path, follow_symlinks=False)
-            else:
-                os.mkdir(path)
-                for dependency_path, child_path in deps:
-                    path_util.copy(dependency_path, child_path, follow_symlinks=False)
+                        # Download the dependency to dependency_path (which is now in the temporary directory).
+                        # TODO (Ashwin): Unify some of the logic here with the code in DependencyManager._store_dependency()
+                        # into common utility functions.
+                        if target_info['type'] == 'directory':
+                            fileobj = self._download_manager.stream_tarred_gzipped_directory(target)
+                            un_tar_directory(fileobj, dependency_path, 'gz')
+                        else:
+                            fileobj = self._download_manager.stream_file(target, gzipped=False)
+                            with open(dependency_path, 'wb') as f:
+                                shutil.copyfileobj(fileobj, f)
+
+                    deps.append((dependency_path, child_path))
+
+                remove_path(path)
+
+                if len(deps) == 1 and deps[0][1] == path:
+                    path_util.copy(deps[0][0], path, follow_symlinks=False)
+                else:
+                    os.mkdir(path)
+                    for dependency_path, child_path in deps:
+                        path_util.copy(dependency_path, child_path, follow_symlinks=False)
 
-            # TODO(Ashwin): fix
             self._model.update_disk_metadata(bundle, bundle_location, enforce_disk_quota=True)
             logger.info('Finished making bundle %s', bundle.uuid)
             self._model.update_bundle(bundle, {'state': State.READY})
